@@ -30,7 +30,18 @@
 #include <sys/sockio.h>
 #endif
 #include <sys/time.h>
+
+#if defined(__DARWIN_ALIGN_POWER) && __DARWIN_ALIGN_POWER
+/* Apple engineering suggested this as a workaround for a bug in OS X
+ * 10.4 (Tiger) that prevented ioctl(..., SIOCGIFCONF, ...) from
+ * working properly in 64 bit mode */
+#pragma options align=power
+#endif
 #include <net/if.h>
+#if defined(__DARWIN_ALIGN_POWER) && __DARWIN_ALIGN_POWER
+#pragma options align=reset
+#endif
+
 #include <netinet/in.h>
 #include <errno.h>
 #include <netdb.h>
