@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -11,11 +11,8 @@
  * LAM/MPI source distribution.
  * 
  * $HEADER$
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	RBD
  *
- *	$Id: getworkdir.c,v 6.5 2002/10/21 04:48:07 brbarret Exp $
+ * $Id: getworkdir.c,v 6.6 2003/07/05 21:50:25 jsquyres Exp $
  * 
  *	Function:	- get the user's current working directory
  *			- dynamically allocated space
@@ -25,17 +22,19 @@
  *	Returns:	- working directory or NULL
  */
 
-#include "lam.h"
+#include <lam_config.h>
 
 #include <stdlib.h>
 #include <unistd.h>
 
+#include <lam.h>
+#include <laminternal.h>
+
 #define PATHSIZE	2048
 
 
 char *
-getworkdir()
-
+getworkdir(void)
 {
 	char		*cwd;		/* working directory name */
 /*
