@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -11,26 +11,27 @@
  * LAM/MPI source distribution.
  * 
  * $HEADER$
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	GDB
  *
- *	$Id: kiproc.c,v 1.2 2002/10/09 20:57:02 brbarret Exp $
+ * $Id: kiproc.c,v 1.3 2003/07/05 21:50:25 jsquyres Exp $
  *
  *	Function:	- stub for stand-alone kernel
  */
 
-#include <portable.h>
+#include <lam_config.h>
+
 #include <unistd.h>
 
+#include <portable.h>
+#include <kreq.h>
+
+
 /*
  *	kboot
  *
  *	Function:	- a stub for the external-only kernel
  */
 void
-kboot()
-
+kboot(int *argc, char **argv)
 {
 }
 
@@ -40,8 +41,7 @@ kboot()
  *	Function:	- a wrapper for getpid() for external procs
  */
 int4
-kbgetpid()
-
+kbgetpid(void)
 {
 	return(getpid());
 }
