@@ -15,7 +15,7 @@
  *	Copyright 1996 The Ohio State University
  *	RBD/NJN/GDB
  *
- *	$Id: mpitr_dtprint.c,v 6.4 2002/10/09 20:57:03 brbarret Exp $
+ *	$Id: mpitr_dtprint.c,v 6.5 2003/07/05 21:50:25 jsquyres Exp $
  * 
  *	Function:	- MPI datatype trace access functions
  */
@@ -35,21 +35,20 @@
  */
 #define DISP(f, x)	((f) ? ((x) & 0xFFFF) : (x))
 
-/*
- * global functions
- */
-void			mpitr_dtypeprint();
 
 /*
  * local functions
  */
-static void		indent();
-static void		print_basic();
-static void		print_datatype();
-static void		print_contig();
-static void		print_hvector();
-static void		print_hindexed();
-static void		print_struct();
+static void print_datatype(struct trdtype **dtrbuf, int level);
+static void print_basic(int dtype, struct trdtype **dtrace, int nlev);
+static void print_contig(struct trdtype **dtrace, int nlev);
+static void print_hvector(int dtype, struct trdtype **dtrace, int nlev, 
+                          int fl_vec);
+static void print_hindexed(int dtype, struct trdtype **dtrace, int nlev, 
+                           int fl_idx);
+static void print_struct(struct trdtype **dtrace, int nlev);
+static void indent(int lev);
+
 
 /*
  * local variables
@@ -95,14 +94,8 @@ static char		*dtbasic[TRDTMAX + 1] = {
  *			- output buffer (NULL means output to stdout)
  */
 void
-mpitr_dtypeprint(dtype, dtrbuf, indent, pfix, outbuf)
-
-int4			dtype;
-char			*dtrbuf;
-int4			indent;			
-char			*pfix;
-char			*outbuf;
-
+mpitr_dtypeprint(int4 dtype, char *dtrbuf, int4 indent, char *pfix, 
+                 char *outbuf)
 {
 	struct trdtype	*p;			/* ptr trace datatype */
 
@@ -158,11 +151,7 @@ char			*outbuf;
  *			- nesting level
  */
 static void
-print_datatype(dtrbuf, level)
-
-struct trdtype		**dtrbuf;
-int			level;
-
+print_datatype(struct trdtype **dtrbuf, int level)
 {
 	int		format;			/* datatype format */
 
@@ -248,12 +237,7 @@ int			level;
  *			- nesting level
  */
 static void
-print_basic(dtype, dtrace, nlev)
-
-int			dtype;
-struct trdtype		**dtrace;
-int			nlev;
-
+print_basic(int dtype, struct trdtype **dtrace, int nlev)
 {
 	nlifconst = 0;
 	indent(nlev);
@@ -276,11 +260,7 @@ int			nlev;
  *			- nesting level
  */
 static void
-print_contig(dtrace, nlev)
-
-struct trdtype		**dtrace;
-int			nlev;
-
+print_contig(struct trdtype **dtrace, int nlev)
 {
 	indent(nlev);
 
@@ -307,13 +287,7 @@ int			nlev;
  *			- vector flag
  */
 static void
-print_hvector(dtype, dtrace, nlev, fl_vec)
-
-int			dtype;
-struct trdtype		**dtrace;
-int			nlev;
-int			fl_vec;
-
+print_hvector(int dtype, struct trdtype **dtrace, int nlev, int fl_vec)
 {
 	indent(nlev);
 
@@ -342,13 +316,7 @@ int			fl_vec;
  *			- indexed flag
  */
 static void
-print_hindexed(dtype, dtrace, nlev, fl_idx)
-
-int			dtype;
-struct trdtype		**dtrace;
-int			nlev;
-int			fl_idx;
-
+print_hindexed(int dtype, struct trdtype **dtrace, int nlev, int fl_idx)
 {
 	int		i;			/* favourite index */
 	int		count;			/* datatype count */
@@ -389,11 +357,7 @@ int			fl_idx;
  *			- nesting level
  */
 static void
-print_struct(dtrace, nlev)
-
-struct trdtype		**dtrace;
-int			nlev;
-
+print_struct(struct trdtype **dtrace, int nlev)
 {
 	int		i;			/* favourite index */
 	int		count;			/* datatype count */
@@ -435,10 +399,7 @@ int			nlev;
  *	Accepts:	- indentation level
  */
 static void
-indent(lev)
-
-int			lev;
-
+indent(int lev)
 {
 	lev *= identchar;
 
