@@ -1,38 +1,18 @@
 /*
- * Copyright 1998-2001, University of Notre Dame.
- * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
- *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ *                         All rights reserved.
+ * Copyright (c) 1998-2001 University of Notre Dame. 
+ *                         All rights reserved.
+ * Copyright (c) 1994-1998 The Ohio State University.  
+ *                         All rights reserved.
  * 
- * This file is part of the Notre Dame LAM implementation of MPI.
+ * This file is part of the LAM/MPI software package.  For license
+ * information, see the LICENSE file in the top level directory of the
+ * LAM/MPI source distribution.
  * 
- * You should have received a copy of the License Agreement for the Notre
- * Dame LAM implementation of MPI along with the software; see the file
- * LICENSE.  If not, contact Office of Research, University of Notre
- * Dame, Notre Dame, IN 46556.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted subject to the conditions specified in the
- * LICENSE file.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
- * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
- * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- * 
- * Additional copyrights may follow.
- * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	GDB
+ * $HEADER$
  *
- *	$Id: rfaux.c,v 6.4 1999/08/26 22:40:44 jsquyres Exp $
+ * $Id: rfaux.c,v 6.9 2003/05/21 19:17:02 jsquyres Exp $
  *
  *	Function:	- useful auxiliary functions for the file runtime
  *			  library
@@ -45,22 +25,26 @@
 #include <ksignal.h>
 #include <net.h>
 #include <typical.h>
+#include <etc_misc.h>
 
 #include <stdlib.h>
 #include <unistd.h>
 
+
 /*
  * external variables
  */
-extern int		errno;		/* error number */
 extern struct fclient	_ufd[FUMAX];	/* client file descriptor table */
 
+
 /*
  * global functions
  */
-int			_fdvalid();	/* check validity of file desc */ 
-int			_makefreq();	/* make a simple filed request */
-char			*_fnparse();	/* parse nodeid from file name */
+int _makefreq(int4 node, int req, int4 param, char *ibuf, int4 ilen, 
+              char *obuf, int4 olen);
+char * _fnparse(char *xname, int4 *node);
+int _fdvalid(int fd);
+
 
 /*
  *	makefreq
@@ -77,18 +61,9 @@ char			*_fnparse();	/* parse nodeid from file name */
  *
  *	Returns:	- reply code from filed
  */
-
 int
-_makefreq(node, req, param, ibuf, ilen, obuf, olen)
-
-int4			node;
-int			req;
-int4			param;
-char			*ibuf;
-int4			ilen;
-char			*obuf;
-int4			olen;
-
+_makefreq(int4 node, int req, int4 param, char *ibuf, int4 ilen, char *obuf,
+	  int4 olen)
 {
 	struct nmsg	nhead;		/* network message desc. */
 	struct freq  	*request;	/* filed request */
@@ -96,13 +71,14 @@ int4			olen;
 	int		mask;		/* signal mask */
 
 
+	LAM_ZERO_ME(nhead);
 	request = (struct freq *) nhead.nh_data;
 	reply = (struct freply *) nhead.nh_data;
 /*
  * Set up the filed request.
  */
 	request->fq_src_node = (node == LOCAL) ? LOCAL : getnodeid();
-	request->fq_src_event = -getpid();
+	request->fq_src_event = -lam_getpid();
 	request->fq_req = (int4) req;
 	request->fq_param0 = param;
 /*
@@ -123,7 +99,7 @@ int4			olen;
 /*
  * Set up the reply message descriptor.
  */
-	nhead.nh_event = -getpid();
+	nhead.nh_event = -lam_getpid();
 	nhead.nh_length = olen;
 	nhead.nh_msg = obuf;
 
@@ -140,6 +116,7 @@ int4			olen;
 	return(reply->fr_ret);
 }
 
+
 /*
  *	fnparse
  *
@@ -151,13 +128,8 @@ int4			olen;
  *
  *	Returns:	- ptr to local file name
  */
-
 char *
-_fnparse(xname, node)
-
-char			*xname;
-int4			*node;
-
+_fnparse(char *xname, int4 *node)
 {
 	char		*cp;		/* search char ptr */
 
@@ -177,6 +149,7 @@ int4			*node;
 	return(xname);
 }
 
+
 /*
  *	fdvalid
  *
@@ -186,10 +159,8 @@ int4			*node;
  *
  *	Returns:	- 0 or ERROR
  */
-
 int
-_fdvalid(fd)
-
+_fdvalid(int fd)
 {
 	if ((fd < 0) || (fd > FUMAX) || (_ufd[fd].fu_tflags == 0)) {
 		errno = EBADF;
