@@ -10,8 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *
- *	$Id: iattr.c,v 1.9.2.1 2002/10/09 19:48:56 brbarret Exp $
+ *	$Id: iattr.c,v 1.11 2002/10/09 20:57:17 brbarret Exp $
  *
  *	Function:	- initializes an IMPID to run an IMPI job
  */
@@ -33,7 +34,7 @@
 #include <mpisys.h>
 #include <impi-defs.h> /* This file supplied by IMPI server package */
 #include <impi.h>
-#include <debug.h>
+#include <lamdebug.h>
 #include <mpi.h>
 
 
