@@ -10,14 +10,16 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *	Ohio Trollius
  *	Copyright 1995 The Ohio State University
  *	GDB
  *
  *	$Log: proc_schema.h,v $
- *	Revision 1.1.2.1  2002/10/09 19:49:00  brbarret
- *	* Updated license header to match current requirements
- *	
+ *	Revision 1.2  2002/10/09 20:57:25  brbarret
+ *	* Update tree to match the latest, greatest, license for LAM/MPI.  And this
+ *	  one is even done properly :)
+ *
  *	Revision 1.1  2000/02/12 00:14:19  jsquyres
  *	Moved share/boot/proc_schema.a to share/include/proc_schema.h 'cause
  *	multiple places include it.
