@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: connect.c,v 6.14.2.1 2003/08/21 06:32:00 brbarret Exp $
+ * $Id: connect.c,v 6.15 2003/08/22 02:33:18 brbarret Exp $
  *
  *	Function:	- connect to an MPI port
  *	Accepts:	- port name
@@ -165,7 +165,7 @@ MPI_Comm_connect(char *port_name, MPI_Info info, int root, MPI_Comm comm,
  * Make a connection.
  */
 		err = connect_to_port(port_name, comm,
-					cid, &svrcid, &svrsize, &svrgps);
+				      cid, &svrcid, &svrsize, &svrgps);
 		if (err != MPI_SUCCESS) {
 			msg[0] = -1;
 			msg[1] = err;
@@ -289,6 +289,11 @@ MPI_Comm_connect(char *port_name, MPI_Info info, int root, MPI_Comm comm,
 
 	LAM_TRACE(lam_tr_cffend(BLKMPICOMMCONNECT, root, comm, 0, 0));
         lam_resetfunc_m(BLKMPICOMMCONNECT);
+	
+	/* Free memory */
+	if (svrgps != NULL)
+	  free((char*) svrgps);
+
 	return(MPI_SUCCESS);
 }
 
