@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1996 The Ohio State University
- *	RBD
+ * $HEADER$
  *
- *	$Id: errfree.c,v 6.6.2.1 2002/10/09 19:49:07 brbarret Exp $
+ * $Id: errfree.c,v 6.8 2002/12/11 19:15:10 jsquyres Exp $
  *
  *	Function:	- free error handler
  *	Accepts:	- ptr error handler
@@ -52,7 +50,7 @@ int MPI_Errhandler_free(MPI_Errhandler* errhdl)
  */
 	if (errhdl == 0 || *errhdl == MPI_ERRHANDLER_NULL) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPIERRFREE, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPIERRFREE, lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 /*
  * Only internal library calls can free predefined handlers.
@@ -60,7 +58,7 @@ int MPI_Errhandler_free(MPI_Errhandler* errhdl)
 	if (lam_numfuncs() == 1 && (*errhdl == MPI_ERRORS_ARE_FATAL
 			|| *errhdl == MPI_ERRORS_RETURN)) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPIERRFREE, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPIERRFREE, lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 /*
  * Free the handler.
@@ -70,7 +68,8 @@ int MPI_Errhandler_free(MPI_Errhandler* errhdl)
 		if (*errhdl == MPI_ERRORS_ARE_FATAL
 				|| *errhdl == MPI_ERRORS_RETURN) {
 			return(lam_errfunc(MPI_COMM_WORLD,
-				BLKMPIERRFREE, lam_mkerr(MPI_ERR_INTERN, 0)));
+					   BLKMPIERRFREE, 
+					   lam_mkerr(MPI_ERR_INTERN, EPERM)));
 		}
 
 		if ((*errhdl)->eh_f77handle >= 0) {
