@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -11,11 +11,8 @@
  * LAM/MPI source distribution.
  * 
  * $HEADER$
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
  *
- *	$Id: spawn_f.c,v 6.7 2002/10/09 20:59:07 brbarret Exp $
+ * $Id: spawn_f.c,v 6.8 2003/07/06 10:44:01 jsquyres Exp $
  *
  *	Function:	- MPI_Comm_spawn F77 wrapper
  */
@@ -25,19 +22,16 @@
 #include <errno.h>
 #include <stdlib.h>
 
-#include <args.h>
+#include <sfh.h>
 #include <blktype.h>
 #include <mpi.h>
 #include <MPISYSF.h>
 #include <mpisys.h>
 
+
 void
-mpi_comm_spawn_(cmd, av, maxp, info, root, comm, icomm, ec, ierr, nc, na)
-
-char			*cmd, *av;
-int			*maxp, *info, *root, *comm, *icomm, *ec, *ierr;
-int			nc, na;
-
+mpi_comm_spawn_(char *cmd, char *av, int *maxp, int *info, int *root, 
+                int *comm, int *icomm, int *ec, int *ierr, int nc, int na)
 {
 	MPI_Comm	intercomm;		/* parent-child intercomm */
 	char		**argv = 0;		/* C argument vector */
@@ -74,7 +68,7 @@ int			nc, na;
  */
 	command = lam_F2C_string(cmd, nc);
 	if (command == 0) {
-		argvfree(argv);
+                sfh_argv_free(argv);
 		lam_setfunc(BLKMPICOMMSPAWN);
 		*ierr = lam_errfunc(GETHDL(*comm), BLKMPICOMMSPAWN,
 					lam_mkerr(MPI_ERR_SPAWN, errno));
@@ -88,6 +82,6 @@ int			nc, na;
 		intercomm->c_f77handle = *icomm;
 	}
 	
-	argvfree(argv);
+	sfh_argv_free(argv);
 	free(command);
 }
