@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	RBD/NJN
+ * $HEADER$
  *
- *	$Id: gexcl.c,v 6.4.2.1 2002/10/09 19:49:08 brbarret Exp $
+ * $Id: gexcl.c,v 6.6 2002/12/11 19:15:11 jsquyres Exp $
  *
  *	Function:	- form new group by exclusion
  *	Accepts:	- original group
@@ -82,13 +80,13 @@ int MPI_Group_excl(MPI_Group g, int n, int *ranks,
  */
 	if (g == MPI_GROUP_NULL) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPIGROUPEXCL, lam_mkerr(MPI_ERR_GROUP, 0)));
+			BLKMPIGROUPEXCL, lam_mkerr(MPI_ERR_GROUP, EINVAL)));
 	}
 
 	if ((png == 0) || ((n > 0) && (ranks == 0))
 			|| (n < 0) || (n > g->g_nprocs)) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPIGROUPEXCL, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPIGROUPEXCL, lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 /*
  * Handle the trivial cases.
@@ -115,7 +113,7 @@ int MPI_Group_excl(MPI_Group g, int n, int *ranks,
 				pi[*r]->p_mode &= ~LAM_PFLAG;
 			}
 			return(lam_errfunc(MPI_COMM_WORLD, BLKMPIGROUPEXCL,
-						lam_mkerr(MPI_ERR_RANK, 0)));
+					   lam_mkerr(MPI_ERR_RANK, EINVAL)));
 		} else {
 			pi[*r]->p_mode |= LAM_PFLAG;
 		}
