@@ -74,6 +74,18 @@ lam_max(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	long		*pllin, *pllout;
 	unsigned long 	*pullin, *pullout;
 #endif
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        LAM_FORTRAN_C_TYPE_INTEGER1 *pfint1in, *pfint1out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        LAM_FORTRAN_C_TYPE_INTEGER2 *pfint2in, *pfint2out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        LAM_FORTRAN_C_TYPE_INTEGER4 *pfint4in, *pfint4out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        LAM_FORTRAN_C_TYPE_INTEGER8 *pfint8in, *pfint8out;
+#endif
 	
 	if (*datatype == MPI_INT || *datatype == MPI_F_INTEGER) {
 		piin = (int *) invec;
@@ -211,6 +223,58 @@ lam_max(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        if (*datatype == MPI_F_INTEGER1) {
+            pfint1in = (LAM_FORTRAN_C_TYPE_INTEGER1 *) invec;
+            pfint1out = (LAM_FORTRAN_C_TYPE_INTEGER1 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint1in, ++pfint1out) {
+                if (*pfint1in > *pfint1out) {
+                    *pfint1out = *pfint1in;
+                }
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        if (*datatype == MPI_F_INTEGER2) {
+            pfint2in = (LAM_FORTRAN_C_TYPE_INTEGER2 *) invec;
+            pfint2out = (LAM_FORTRAN_C_TYPE_INTEGER2 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint2in, ++pfint2out) {
+                if (*pfint2in > *pfint2out) {
+                    *pfint2out = *pfint2in;
+                }
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        if (*datatype == MPI_F_INTEGER4) {
+            pfint4in = (LAM_FORTRAN_C_TYPE_INTEGER4 *) invec;
+            pfint4out = (LAM_FORTRAN_C_TYPE_INTEGER4 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint4in, ++pfint4out) {
+                if (*pfint4in > *pfint4out) {
+                    *pfint4out = *pfint4in;
+                }
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        if (*datatype == MPI_F_INTEGER8) {
+            pfint8in = (LAM_FORTRAN_C_TYPE_INTEGER8 *) invec;
+            pfint8out = (LAM_FORTRAN_C_TYPE_INTEGER8 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint8in, ++pfint8out) {
+                if (*pfint8in > *pfint8out) {
+                    *pfint8out = *pfint8in;
+                }
+            }
+            return;
+        }
+#endif
+
 	(void) lam_errfunc(MPI_COMM_WORLD,
 			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
@@ -239,6 +303,18 @@ lam_min(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	long		*pllin, *pllout;
 	unsigned long 	*pullin, *pullout;
 #endif
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        LAM_FORTRAN_C_TYPE_INTEGER1 *pfint1in, *pfint1out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        LAM_FORTRAN_C_TYPE_INTEGER2 *pfint2in, *pfint2out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        LAM_FORTRAN_C_TYPE_INTEGER4 *pfint4in, *pfint4out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        LAM_FORTRAN_C_TYPE_INTEGER8 *pfint8in, *pfint8out;
+#endif
 	
 	if (*datatype == MPI_INT || *datatype == MPI_F_INTEGER) {
 		piin = (int *) invec;
@@ -376,6 +452,58 @@ lam_min(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        if (*datatype == MPI_F_INTEGER1) {
+            pfint1in = (LAM_FORTRAN_C_TYPE_INTEGER1 *) invec;
+            pfint1out = (LAM_FORTRAN_C_TYPE_INTEGER1 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint1in, ++pfint1out) {
+                if (*pfint1in < *pfint1out) {
+                    *pfint1out = *pfint1in;
+                }
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        if (*datatype == MPI_F_INTEGER2) {
+            pfint2in = (LAM_FORTRAN_C_TYPE_INTEGER2 *) invec;
+            pfint2out = (LAM_FORTRAN_C_TYPE_INTEGER2 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint2in, ++pfint2out) {
+                if (*pfint2in < *pfint2out) {
+                    *pfint2out = *pfint2in;
+                }
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        if (*datatype == MPI_F_INTEGER4) {
+            pfint4in = (LAM_FORTRAN_C_TYPE_INTEGER4 *) invec;
+            pfint4out = (LAM_FORTRAN_C_TYPE_INTEGER4 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint4in, ++pfint4out) {
+                if (*pfint4in < *pfint4out) {
+                    *pfint4out = *pfint4in;
+                }
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        if (*datatype == MPI_F_INTEGER8) {
+            pfint8in = (LAM_FORTRAN_C_TYPE_INTEGER8 *) invec;
+            pfint8out = (LAM_FORTRAN_C_TYPE_INTEGER8 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint8in, ++pfint8out) {
+                if (*pfint8in < *pfint8out) {
+                    *pfint8out = *pfint8in;
+                }
+            }
+            return;
+        }
+#endif
+
 	(void) lam_errfunc(MPI_COMM_WORLD,
 			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
@@ -407,6 +535,18 @@ lam_sum(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	long		*pllin, *pllout;
 	unsigned long 	*pullin, *pullout;
 #endif
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        LAM_FORTRAN_C_TYPE_INTEGER1 *pfint1in, *pfint1out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        LAM_FORTRAN_C_TYPE_INTEGER2 *pfint2in, *pfint2out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        LAM_FORTRAN_C_TYPE_INTEGER4 *pfint4in, *pfint4out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        LAM_FORTRAN_C_TYPE_INTEGER8 *pfint8in, *pfint8out;
+#endif
 
 	if (*datatype == MPI_INT || *datatype == MPI_F_INTEGER) {
 		piin = (int *) invec;
@@ -553,6 +693,50 @@ lam_sum(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        if (*datatype == MPI_F_INTEGER1) {
+            pfint1in = (LAM_FORTRAN_C_TYPE_INTEGER1 *) invec;
+            pfint1out = (LAM_FORTRAN_C_TYPE_INTEGER1 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint1in, ++pfint1out) {
+                *pfint1out += *pfint1in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        if (*datatype == MPI_F_INTEGER2) {
+            pfint2in = (LAM_FORTRAN_C_TYPE_INTEGER2 *) invec;
+            pfint2out = (LAM_FORTRAN_C_TYPE_INTEGER2 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint2in, ++pfint2out) {
+                *pfint2out += *pfint2in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        if (*datatype == MPI_F_INTEGER4) {
+            pfint4in = (LAM_FORTRAN_C_TYPE_INTEGER4 *) invec;
+            pfint4out = (LAM_FORTRAN_C_TYPE_INTEGER4 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint4in, ++pfint4out) {
+                *pfint4out += *pfint4in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        if (*datatype == MPI_F_INTEGER8) {
+            pfint8in = (LAM_FORTRAN_C_TYPE_INTEGER8 *) invec;
+            pfint8out = (LAM_FORTRAN_C_TYPE_INTEGER8 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint8in, ++pfint8out) {
+                *pfint8out += *pfint8in;
+            }
+            return;
+        }
+#endif
+
 	(void) lam_errfunc(MPI_COMM_WORLD,
 			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
@@ -588,6 +772,18 @@ lam_prod(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	long		*pllin, *pllout;
 	unsigned long 	*pullin, *pullout;
 #endif
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        LAM_FORTRAN_C_TYPE_INTEGER1 *pfint1in, *pfint1out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        LAM_FORTRAN_C_TYPE_INTEGER2 *pfint2in, *pfint2out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        LAM_FORTRAN_C_TYPE_INTEGER4 *pfint4in, *pfint4out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        LAM_FORTRAN_C_TYPE_INTEGER8 *pfint8in, *pfint8out;
+#endif
 
 	if (*datatype == MPI_INT || *datatype == MPI_F_INTEGER) {
 		piin = (int *) invec;
@@ -741,6 +937,50 @@ lam_prod(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        if (*datatype == MPI_F_INTEGER1) {
+            pfint1in = (LAM_FORTRAN_C_TYPE_INTEGER1 *) invec;
+            pfint1out = (LAM_FORTRAN_C_TYPE_INTEGER1 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint1in, ++pfint1out) {
+                *pfint1out *= *pfint1in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        if (*datatype == MPI_F_INTEGER2) {
+            pfint2in = (LAM_FORTRAN_C_TYPE_INTEGER2 *) invec;
+            pfint2out = (LAM_FORTRAN_C_TYPE_INTEGER2 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint2in, ++pfint2out) {
+                *pfint2out *= *pfint2in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        if (*datatype == MPI_F_INTEGER4) {
+            pfint4in = (LAM_FORTRAN_C_TYPE_INTEGER4 *) invec;
+            pfint4out = (LAM_FORTRAN_C_TYPE_INTEGER4 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint4in, ++pfint4out) {
+                *pfint4out *= *pfint4in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        if (*datatype == MPI_F_INTEGER8) {
+            pfint8in = (LAM_FORTRAN_C_TYPE_INTEGER8 *) invec;
+            pfint8out = (LAM_FORTRAN_C_TYPE_INTEGER8 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint8in, ++pfint8out) {
+                *pfint8out *= *pfint8in;
+            }
+            return;
+        }
+#endif
+
 	(void) lam_errfunc(MPI_COMM_WORLD,
 			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
@@ -1143,6 +1383,18 @@ lam_band(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	long		*pllin, *pllout;
 	unsigned long 	*pullin, *pullout;
 #endif
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        LAM_FORTRAN_C_TYPE_INTEGER1 *pfint1in, *pfint1out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        LAM_FORTRAN_C_TYPE_INTEGER2 *pfint2in, *pfint2out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        LAM_FORTRAN_C_TYPE_INTEGER4 *pfint4in, *pfint4out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        LAM_FORTRAN_C_TYPE_INTEGER8 *pfint8in, *pfint8out;
+#endif
 
 	if ((*datatype == MPI_INT) || (*datatype == MPI_F_LOGICAL)
 			|| (*datatype == MPI_F_INTEGER)) {
@@ -1236,6 +1488,50 @@ lam_band(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        if (*datatype == MPI_F_INTEGER1) {
+            pfint1in = (LAM_FORTRAN_C_TYPE_INTEGER1 *) invec;
+            pfint1out = (LAM_FORTRAN_C_TYPE_INTEGER1 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint1in, ++pfint1out) {
+                *pfint1out &= *pfint1in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        if (*datatype == MPI_F_INTEGER2) {
+            pfint2in = (LAM_FORTRAN_C_TYPE_INTEGER2 *) invec;
+            pfint2out = (LAM_FORTRAN_C_TYPE_INTEGER2 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint2in, ++pfint2out) {
+                *pfint2out &= *pfint2in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        if (*datatype == MPI_F_INTEGER4) {
+            pfint4in = (LAM_FORTRAN_C_TYPE_INTEGER4 *) invec;
+            pfint4out = (LAM_FORTRAN_C_TYPE_INTEGER4 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint4in, ++pfint4out) {
+                *pfint4out &= *pfint4in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        if (*datatype == MPI_F_INTEGER8) {
+            pfint8in = (LAM_FORTRAN_C_TYPE_INTEGER8 *) invec;
+            pfint8out = (LAM_FORTRAN_C_TYPE_INTEGER8 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint8in, ++pfint8out) {
+                *pfint8out &= *pfint8in;
+            }
+            return;
+        }
+#endif
+
 	(void) lam_errfunc(MPI_COMM_WORLD,
 			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
@@ -1258,6 +1554,18 @@ lam_bor(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	long		*pllin, *pllout;
 	unsigned long 	*pullin, *pullout;
 #endif
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        LAM_FORTRAN_C_TYPE_INTEGER1 *pfint1in, *pfint1out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        LAM_FORTRAN_C_TYPE_INTEGER2 *pfint2in, *pfint2out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        LAM_FORTRAN_C_TYPE_INTEGER4 *pfint4in, *pfint4out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        LAM_FORTRAN_C_TYPE_INTEGER8 *pfint8in, *pfint8out;
+#endif
 
 	if ((*datatype == MPI_INT) || (*datatype == MPI_F_LOGICAL)
 			|| (*datatype == MPI_F_INTEGER)) {
@@ -1351,6 +1659,50 @@ lam_bor(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        if (*datatype == MPI_F_INTEGER1) {
+            pfint1in = (LAM_FORTRAN_C_TYPE_INTEGER1 *) invec;
+            pfint1out = (LAM_FORTRAN_C_TYPE_INTEGER1 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint1in, ++pfint1out) {
+                *pfint1out |= *pfint1in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        if (*datatype == MPI_F_INTEGER2) {
+            pfint2in = (LAM_FORTRAN_C_TYPE_INTEGER2 *) invec;
+            pfint2out = (LAM_FORTRAN_C_TYPE_INTEGER2 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint2in, ++pfint2out) {
+                *pfint2out |= *pfint2in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        if (*datatype == MPI_F_INTEGER4) {
+            pfint4in = (LAM_FORTRAN_C_TYPE_INTEGER4 *) invec;
+            pfint4out = (LAM_FORTRAN_C_TYPE_INTEGER4 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint4in, ++pfint4out) {
+                *pfint4out |= *pfint4in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        if (*datatype == MPI_F_INTEGER8) {
+            pfint8in = (LAM_FORTRAN_C_TYPE_INTEGER8 *) invec;
+            pfint8out = (LAM_FORTRAN_C_TYPE_INTEGER8 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint8in, ++pfint8out) {
+                *pfint8out |= *pfint8in;
+            }
+            return;
+        }
+#endif
+
 	(void) lam_errfunc(MPI_COMM_WORLD,
 			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
@@ -1373,6 +1725,18 @@ lam_bxor(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	long		*pllin, *pllout;
 	unsigned long 	*pullin, *pullout;
 #endif
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        LAM_FORTRAN_C_TYPE_INTEGER1 *pfint1in, *pfint1out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        LAM_FORTRAN_C_TYPE_INTEGER2 *pfint2in, *pfint2out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        LAM_FORTRAN_C_TYPE_INTEGER4 *pfint4in, *pfint4out;
+#endif
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        LAM_FORTRAN_C_TYPE_INTEGER8 *pfint8in, *pfint8out;
+#endif
 
 	if ((*datatype == MPI_INT) || (*datatype == MPI_F_LOGICAL)
 			|| (*datatype == MPI_F_INTEGER)) {
@@ -1466,6 +1830,50 @@ lam_bxor(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
+#if LAM_SIZEOF_FORTRAN_INTEGER1
+        if (*datatype == MPI_F_INTEGER1) {
+            pfint1in = (LAM_FORTRAN_C_TYPE_INTEGER1 *) invec;
+            pfint1out = (LAM_FORTRAN_C_TYPE_INTEGER1 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint1in, ++pfint1out) {
+                *pfint1out ^= *pfint1in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER2
+        if (*datatype == MPI_F_INTEGER2) {
+            pfint2in = (LAM_FORTRAN_C_TYPE_INTEGER2 *) invec;
+            pfint2out = (LAM_FORTRAN_C_TYPE_INTEGER2 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint2in, ++pfint2out) {
+                *pfint2out ^= *pfint2in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER4
+        if (*datatype == MPI_F_INTEGER4) {
+            pfint4in = (LAM_FORTRAN_C_TYPE_INTEGER4 *) invec;
+            pfint4out = (LAM_FORTRAN_C_TYPE_INTEGER4 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint4in, ++pfint4out) {
+                *pfint4out ^= *pfint4in;
+            }
+            return;
+        }
+#endif
+
+#if LAM_SIZEOF_FORTRAN_INTEGER8
+        if (*datatype == MPI_F_INTEGER8) {
+            pfint8in = (LAM_FORTRAN_C_TYPE_INTEGER8 *) invec;
+            pfint8out = (LAM_FORTRAN_C_TYPE_INTEGER8 *) inoutvec;
+            for (i = 0; i < *len; ++i, ++pfint8in, ++pfint8out) {
+                *pfint8out ^= *pfint8in;
+            }
+            return;
+        }
+#endif
+
 	(void) lam_errfunc(MPI_COMM_WORLD,
 			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
@@ -1750,6 +2158,10 @@ lam_replace(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 {
 	if (*datatype == MPI_INT
 		|| *datatype == MPI_F_INTEGER
+		|| *datatype == MPI_F_INTEGER1
+		|| *datatype == MPI_F_INTEGER2
+		|| *datatype == MPI_F_INTEGER4
+		|| *datatype == MPI_F_INTEGER8
 		|| *datatype == MPI_LONG
 		|| *datatype == MPI_BYTE
 		|| *datatype == MPI_UNSIGNED_CHAR
