@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: namepub.c,v 6.5.2.1 2002/10/09 19:49:11 brbarret Exp $
+ * $Id: namepub.c,v 6.7 2002/12/11 19:15:13 jsquyres Exp $
  *
  *	Function:	- publish a port/service pair
  *	Accepts:	- service name
@@ -73,7 +71,7 @@ MPI_Publish_name(char *service_name, MPI_Info info, char *port_name)
  */
 	if (port_name == 0 || service_name == 0) {
 		return(lam_errfunc(MPI_COMM_WORLD, BLKMPIPUBLISHNAME,
-				lam_mkerr(MPI_ERR_ARG, 0)));
+				lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 /*
  * Publish name.
