@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1996 The Ohio State University
- *	RBD
+ * $HEADER$
  *
- *	$Id: opfree.c,v 6.4.2.1 2002/10/09 19:49:12 brbarret Exp $
+ * $Id: opfree.c,v 6.6 2002/12/11 19:15:13 jsquyres Exp $
  *
  *	Function:	- free a reduction operator
  *	Accepts:	- operator (inout)
@@ -58,7 +56,7 @@ int MPI_Op_free(MPI_Op *op)
 	if (op == 0 || (*op == MPI_OP_NULL)
 			|| ((*op)->op_flags & LAM_PREDEF)) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPIOPFREE, lam_mkerr(MPI_ERR_OP, 0)));
+			BLKMPIOPFREE, lam_mkerr(MPI_ERR_OP, EINVAL)));
 	}
 
 /*
