@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1996 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: portopen.c,v 6.7.2.1 2002/10/09 19:49:12 brbarret Exp $
+ * $Id: portopen.c,v 6.10 2003/05/31 22:28:50 jsquyres Exp $
  *
  *	Function:	- open an MPI port
  *	Accepts:	- info
@@ -32,6 +30,7 @@
 #include <mpisys.h>
 #include <rpisys.h>
 #include <terror.h>
+#include <etc_misc.h>
 
 /*@
    MPI_Open_port - Establish a server port for client MPI connections
@@ -77,7 +76,7 @@ MPI_Open_port(MPI_Info info, char* port_name)
 	
 	if (al_find(lam_ports, &port) != 0) {
 		return(lam_errfunc(MPI_COMM_WORLD, BLKMPIOPENPORT,
-					lam_mkerr(MPI_ERR_INTERN, 0)));
+					lam_mkerr(MPI_ERR_INTERN, EINVAL)));
 	}
 	if (al_insert(lam_ports, &port) == 0) {
 		return(lam_errfunc(MPI_COMM_WORLD, BLKMPIOPENPORT,
@@ -87,7 +86,7 @@ MPI_Open_port(MPI_Info info, char* port_name)
  * Set return value.  The MPI_MAX_PORT_NAME probably isn't necessary, but
  * hey, let's be safe.
  */
-	strncpy(port_name, port.prt_name, MPI_MAX_PORT_NAME);
+	lam_strncpy(port_name, port.prt_name, MPI_MAX_PORT_NAME);
 	
         lam_resetfunc(BLKMPIOPENPORT);
 	return(MPI_SUCCESS);
