@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: wwait.c,v 1.2.2.1 2002/10/09 19:49:16 brbarret Exp $
+ * $Id: wwait.c,v 1.4 2002/12/11 19:15:15 jsquyres Exp $
  *
  *	Function:	- one sided window wait
  *	Accepts:	- window
@@ -69,14 +67,14 @@ int MPI_Win_wait(MPI_Win win)
  */
 	if (win == MPI_WIN_NULL) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPIWINWAIT, lam_mkerr(MPI_ERR_WIN, 0)));
+			BLKMPIWINWAIT, lam_mkerr(MPI_ERR_WIN, EINVAL)));
 	}
 
 	comm = win->w_comm;
 
 	if (!(win->w_flags & LAM_WFPOSTED)) {
 		return(lam_errfunc(comm,
-			BLKMPIWINWAIT, lam_mkerr(MPI_ERR_EPOCH, 0)));
+			BLKMPIWINWAIT, lam_mkerr(MPI_ERR_EPOCH, EINVAL)));
 	}
 /*
  * Loop until everybody has completed.
