@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,7 +12,7 @@
  *
  * $HEADER$
  *
- * $Id: ssi_close.c,v 1.2.2.2 2004/02/18 23:09:46 vsahay Exp $
+ * $Id: ssi_close.c,v 1.5 2003/11/15 13:32:07 jsquyres Exp $
  *
  */
 
@@ -24,23 +24,15 @@
 #include <lam-ssi.h>
 
 
-/*
- * local variables
- */
-
-static int finalized = 0;
-
-
 /*
  * Main SSI shutdown.
  */
 int
 lam_ssi_base_close(void)
 {
-  /* Were we already here?  We really only need to do this once. */
+  /* Clear out all the registered SSI params */
 
-  if (finalized == 1)
-    return 0;
+  lam_ssi_base_param_finalize();
 
   /* If we had a stream open, close it */
 
@@ -52,8 +44,15 @@ lam_ssi_base_close(void)
     lam_ssi_did = -1;
   }
 
+  /* Release all resources allocated by hostmap translation */
+
+  lam_ssi_base_hostmap_finalize();
+
+  /* Close down the module registry */
+
+  lam_ssi_base_module_registry_finalize();
+
   /* All done */
 
-  finalized = 1;
   return 0;
 }
