@@ -725,6 +725,81 @@ alg_vector(struct lamnode *nodes, int nnodes,
 }
 
 
+/* Josh: This function only needed if bproc_vexecmove() isn't native */
+#ifndef HAVE_BPROC_VEXECMOVE
+/*
+ * bproc_vexecmove
+ *
+ * DESCRIPTION:
+ *  Josh: Scyld fork of BProc was based on 3.1.9, therefore it doesn't
+ *  have bproc_vexecmove(), so we simulate it by using
+ *  bproc_execmove() and manually iterating through the vector ourselves
+ */
+static void
+bproc_vexecmove(int nnodes, int *nodes, int *pid, const char *cmd,
+                char * const argv[], char * const envp[])
+{
+	const char 	*prefix = "bproc:vexecmove:";
+	int			i;
+	int			thisnode;
+
+	/* don't need bproc if executing on the node we're already on */
+	thisnode = bproc_currnode();
+	for (i = 0; i < nnodes; i ++)
+	{
+		/* create each child process */
+		pid[i] = fork();
+		if (0 == pid[i])
+		{
+			/* on child */
+			if (lam_ssi_boot_verbose >= 1)
+			{
+				/* log success only if verbose turned on */
+				lam_debug(lam_ssi_boot_did, "%s index %d, node %d, child about to exec %s\n",
+				 prefix, i, nodes[i], cmd);
+			}
+			
+			/* choose either local or remote exec */
+			if (nodes[i] != thisnode && nodes[i] != BPROC_NODE_SELF)
+			{
+				/* different node, use bproc to reach it */
+				bproc_execmove(nodes[i], cmd, argv, envp);
+			}
+			else
+			{
+				/* same node, use ordinary execve() instead of bproc */
+				execve(cmd, argv, envp);
+			}
+			
+			/* error, should not be here anymore */
+			lam_debug(lam_ssi_boot_did, "%s index %d, node %d, child FAILED exec of %s: %s\n",
+			 prefix, i, nodes[i], cmd, strerror(errno));
+
+			/* nothing more the child can do */
+			exit(LAMERROR);
+		}
+		else if (pid[i] != -1)
+		{
+			/* on parent */
+			if (lam_ssi_boot_verbose >= 1)
+			{
+				/* log success only if verbose turned on */
+				lam_debug(lam_ssi_boot_did,
+				 "%s index %d, node %d, parent did fork of child as pid %d\n",
+				 prefix, i, nodes[i], pid[i]);
+			}
+		}
+		else
+		{
+			/* fork failed, still on parent */
+			lam_debug(lam_ssi_boot_did, "%s index %d, node %d, parent FAILED to fork: %s\n",
+			 prefix, i, nodes[i], strerror(errno));
+		}
+	}
+}
+#endif /* HAVE_BPROC_VEXECMOVE */
+
+
 /*
  * Use bproc_vexecmove to loop over starting each app on a vector of
  * nodes
