@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,7 +12,7 @@
  *
  * $HEADER$
  *
- * $Id: ssi_boot_globus.c,v 1.4.2.1 2004/01/02 00:11:49 jsquyres Exp $
+ * $Id: ssi_boot_globus.c,v 1.7 2004/01/02 00:11:42 jsquyres Exp $
  *
  *	Function:	- globus boot SSI module
  */
@@ -38,16 +38,17 @@
 #include <laminternal.h>
 #include <lam-ssi-boot-globus.h>
 
+
 /*
  * Access the environment
  */
-
 extern char **environ;
 
+
 /*
  * Local variables
  */
-
+static int param_priority;
 static ELEM search;
 static ELEM *prefix_keyval;
 static char prefix_key[] = "prefix";
@@ -57,15 +58,7 @@ static char *bhost = NULL;
 static int origin = -1;
 static struct lamnode origin_node;
 static char *globuslocation;
-
-/* 
- * Local function
- */
-
-static void append_path(char *prefix, char *dir, char *fname,
-			int *argcnt, char ***arglist);
-
-static const lam_ssi_boot_actions_1_0_0_t module_actions = {
+static const lam_ssi_boot_actions_1_1_0_t module_actions = {
 
   /* Boot SSI API functions */
 
@@ -82,6 +75,7 @@ static const lam_ssi_boot_actions_1_0_0_t module_actions = {
   /* The rest of the functions use the generic TCP protocol functions
      that are common to all boot SSI's */
 
+  lam_ssi_boot_base_get_nodeid,
   lam_ssi_boot_base_open_srv_connection,
   lam_ssi_boot_base_send_lamd_info,
   lam_ssi_boot_base_receive_lamd_info,
@@ -92,6 +86,20 @@ static const lam_ssi_boot_actions_1_0_0_t module_actions = {
 };
 
 
+/*
+ * Open the module and register an SSI parameter
+ */
+int
+lam_ssi_boot_globus_open_module(OPT *ad)
+{
+  param_priority =
+    lam_ssi_base_param_register_int("boot", "globus", "priority", NULL, 
+                                    LAM_SSI_BOOT_GLOBUS_DEFAULT ? 75 : 3);
+
+  return 1;
+}
+
+
 /*
  * Initialize the globus boot SSI
  *
@@ -106,9 +114,8 @@ lam_ssi_boot_globus_init(lam_ssi_boot_location_t where, int *priority)
 
   /* Check this module's priority before doing anything else */
   
-  if (lam_ssi_boot_base_check_priority("globus", 3,
-				       LAM_SSI_BOOT_GLOBUS_DEFAULT,
-				       priority) < 0)
+  *priority = lam_ssi_base_param_lookup_int(param_priority);
+  if (*priority < 0)
     return NULL;
  
   /* Do we have Globus installed */
@@ -187,6 +194,7 @@ lam_ssi_boot_globus_allocate_nodes(struct lamnode **nodes, int *nnodes,
   char *listhost;
   ELEM *host_keyval;
   ELEM keyvalue;
+  char *prefix = NULL;
 
   /* If we got a bhost above, parse the system file */
 
@@ -208,27 +216,28 @@ lam_ssi_boot_globus_allocate_nodes(struct lamnode **nodes, int *nnodes,
     if (n[i].lnd_nodeid != NOTNODEID) {
 
       /* get the prefix if there, else throw error */
+      /*
+	search.key = prefix_key;
+	prefix_keyval = al_find(n[i].lnd_keyval, &search);
 
-      search.key = prefix_key;
-      prefix_keyval = al_find(n[i].lnd_keyval, &search);
+	if (prefix_keyval != NULL) {
+      */
+      
+      prefix = lam_ssi_boot_base_find_prefix(n[i].lnd_keyval);
 
-      if (prefix_keyval != NULL) {
-	if (lam_ssi_boot_verbose >= 1) 
-	  lam_debug(lam_ssi_boot_did,
-		    "globus:   n%d %s (cpu=%d) (prefix=%s)", 
-		    i, n[i].lnd_hname, n[i].lnd_ncpus, prefix_keyval->value);
-      } else {
-	if (lam_ssi_boot_verbose >= 1) 
-	  lam_debug(lam_ssi_boot_did,
-		    "globus:   n%d %s (cpu=%d) (prefix=NULL)", 
-		    i, n[i].lnd_hname, n[i].lnd_ncpus);
-	
+      if (lam_ssi_boot_verbose >= 1) 
+	lam_debug(lam_ssi_boot_did,
+		  "globus:   n%d %s (cpu=%d) (prefix=%s)", 
+		  i, n[i].lnd_hname, n[i].lnd_ncpus, 
+		  (prefix ? prefix : "NULL"));
+
+      if (!prefix) {
 	show_help_file("lam-ssi-boot-globus-helpfile", "boot",
-		       "no-prefix", "The boot SSI globus module",
-		       bhost, n[i].lnd_hname);
+			 "no-prefix", "The boot SSI globus module",
+			 bhost, n[i].lnd_hname);
 	return LAMERROR;
       }
-
+    
       /* look if we got contact string other than hostname format */
 	
       search.key = host_key;
@@ -535,7 +544,10 @@ lam_ssi_boot_globus_start_rte_proc(struct lamnode *node,
 
     /* build the hboot path */
 
-    append_path(prefix, "bin", "hboot", &argc, &argv);
+    ret = lam_ssi_boot_base_append_path(prefix, "bin", "hboot", &argc, &argv);
+    if (ret == LAMERROR)
+      return ret;
+    
     sfh_argv_add(&argc, &argv, "-t");
 
     /* If we specify a conf file, use that.  Otherwise, use the
@@ -544,8 +556,12 @@ lam_ssi_boot_globus_start_rte_proc(struct lamnode *node,
     sfh_argv_add(&argc, &argv, "-c");
     if (ao_taken(lam_ssi_boot_optd, "c"))
       sfh_argv_add(&argc, &argv, ao_param(lam_ssi_boot_optd, "c", 0, 0));
-    else 
-      append_path(prefix, "etc", "lam-conf.lamd", &argc, &argv);
+    else {
+      ret = lam_ssi_boot_base_append_path(prefix, "etc", "lam-conf.lamd", 
+					  &argc, &argv);
+      if (ret == LAMERROR)
+	return ret;
+    }
 
     /* Common params */
 
@@ -593,8 +609,11 @@ lam_ssi_boot_globus_start_rte_proc(struct lamnode *node,
       lam_debug(lam_ssi_boot_did, "globus: starting recon on (%s)",
 		node->lnd_hname);
 
-    append_path(prefix, "bin", DEFTTKILL, &argc, &argv);
-
+    ret = lam_ssi_boot_base_append_path(prefix, "bin", DEFTTKILL, 
+					&argc, &argv);
+    if (ret == LAMERROR)
+      return ret;
+    
     sfh_argv_add(&argc, &argv, "-N");
 
     /* Get session suffix */
@@ -614,7 +633,10 @@ lam_ssi_boot_globus_start_rte_proc(struct lamnode *node,
       lam_debug(lam_ssi_boot_did, "globus: starting wipe on (%s)",
 		node->lnd_hname);
 
-    append_path(prefix, "bin", DEFTTKILL, &argc, &argv);
+    ret = lam_ssi_boot_base_append_path(prefix, "bin", DEFTTKILL, 
+					&argc, &argv);
+    if (ret == LAMERROR)
+      return ret;
 
     /* Get session suffix */
     lam_tmpdir_init(NULL, NULL);
@@ -647,17 +669,3 @@ lam_ssi_boot_globus_start_rte_proc(struct lamnode *node,
 
   return ret;
 }
-
-
-static void
-append_path(char *prefix, char *dir, char *fname, int *argcnt, char ***arglist)
-{
-  int path_len;
-  char *path_var;
-
-  path_len = strlen(prefix) + strlen(dir) + strlen(fname) + 3;
-  path_var = (char *) malloc (path_len * sizeof(char));
-  snprintf(path_var, path_len, "%s/%s/%s", prefix, dir, fname);
-  sfh_argv_add(argcnt, arglist, path_var);
-  free(path_var);
-}
