@@ -504,7 +504,7 @@ lam_ssi_boot_rsh_start_rte_proc(struct lamnode *node,
 #endif
   char *session_suffix = NULL;
   ELEM search;
-  ELEM *prefix_keyval;
+  ELEM *prefix_keyval = NULL;
   char prefix_key[] = "prefix";
 
   local_lamprefix = lamprefix;
