@@ -270,7 +270,6 @@ lam_ssi_crlam_self_finalize(void)
 
   /* Wait for the thread to exit */
 
-  printf ("Finalize inside CRLAM: Going to wait for the Thread to exit\n");
   if (lam_thread_join(crlam_self_thread, NULL) == -1) {
     show_help_file("lam-ssi-crlam-self-helpfile", "cr-self-init",
         "thread-join-fail", NULL);
@@ -362,7 +361,6 @@ lam_ssi_crlam_self_checkpoint(void)
       lam_debug(lam_ssi_cr_did, "Couldn't make app schema!");
     return LAMERROR;
   }
-  printf("Done making the appschema\n%s\n", chkpnt_app_schema);
 
   return 0; 
 }
@@ -434,12 +432,6 @@ signal_handler(int sig)
     close(crlam_self_filedesc[1]);
     /* JMS: Need to do something here...*/
   }
-
-  /* JMS: Debug junk */
-  {
-    char msg[] = "Inside CRLAM's *signal* handler\n";
-    write(1, msg, sizeof(msg));
-  }
 }
 
 
