@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: ssi_rpi_gm_long.c,v 1.15.2.4 2004/03/12 01:19:12 vsahay Exp $
+ * $Id: ssi_rpi_gm_long.c,v 1.29 2004/02/15 14:02:05 jsquyres Exp $
  *
  *	Function: - everything to do with long messages.  Functions in
  *	this file are marked with "<SENDER>" or "<RECEIVER>" to
@@ -43,23 +43,27 @@
  * private functions
  */
 
-static int send_long_first(MPI_Request req);
-static int send_long_second(MPI_Request req);
-static int send_long_ack(MPI_Request req);
-static int send_all_done_env(MPI_Request req);
+static int sender_send_env(MPI_Request req);
+static void sender_send_env_callback(struct gm_port *port, void *context, 
+                                     gm_status_t status);
 
-static void long_send_first_env_callback(struct gm_port *port, void *context, 
-					 gm_status_t status);
-static void long_send_ack_callback(struct gm_port *port, void *context, 
-				   gm_status_t status);
-static void long_send_second_env_callback(struct gm_port *port, void *context, 
-					  gm_status_t status);
-static void long_send_body_unpin_callback(struct gm_port *port, void *context, 
-					  gm_status_t status);
-static void long_send_body_no_unpin_callback(struct gm_port *port, 
-					     void *context, 
-					     gm_status_t status);
-static int long_receive_body(MPI_Request req);
+static int receiver_send_ack(MPI_Request req);
+static void receiver_send_ack_callback(struct gm_port *port, void *context, 
+                                       gm_status_t status);
+
+static int sender_recv_ack(MPI_Request req, char *raw_env_data);
+static int sender_put_body(MPI_Request req);
+static void sender_put_body_unpin_callback(struct gm_port *port, 
+                                           void *context, gm_status_t status);
+static void sender_put_body_no_unpin_callback(struct gm_port *port, 
+                                              void *context, 
+                                              gm_status_t status);
+
+static int sender_send_all_done(MPI_Request req);
+static void  sender_send_all_done_callback(struct gm_port *port, 
+                                           void *context, gm_status_t status);
+
+static int receiver_recv_body(MPI_Request req);
 
 
 /*
@@ -93,17 +97,7 @@ lam_ssi_rpi_gm_long_send(MPI_Request req)
   req->rq_state = LAM_RQSACTIVE;
   req->rq_rpi->is_advancing = 1;
 
-  /* Queue this request up to receive the ACK.  We must do this
-     *before* the actual send to prevent a race condition -- there's
-     no sense in having to do unexpected receives for ACKs.  i.e.,
-     *guarantee* that the ACK will be expected by posting it before we
-     do the actual send.  It's ok to do this, even if we don't know if
-     the message will actually be sent now or later because the ACK
-     request are queued up in order. */
-
-  req->rq_rpi->recv_advance_fn = lam_ssi_rpi_gm_long_send_second;
-  if (lam_ssi_rpi_gm_add_ack(req->rq_proc->p_rpi, req) != 0)
-    return LAMERROR;
+  req->rq_rpi->recv_advance_fn = sender_recv_ack;
 
   /* We only need one send token here because we're only going to send
      out the envelope right now.  If we have to queue it up, mark the
@@ -111,9 +105,11 @@ lam_ssi_rpi_gm_long_send(MPI_Request req)
   /* NOTE: The envelope that is sent has a cr_rank of *my* rank (in
      this communicator) -- i.e., who the long message is *from*. */
 
-  if (!LAM_SSI_RPI_GM_SEND_QUEUE_EMPTY() ||
-      lam_ssi_rpi_gm_stokens < 1) {
-    req->rq_rpi->send_advance_fn = send_long_first;
+  if (!LAM_SSI_RPI_GM_SEND_QUEUE_EMPTY() || 
+      (PUB(stokens) < 1) || 
+      (req->rq_proc->p_rpi->num_dropped_sends > 0) ||
+      (1 == lam_ssi_rpi_gm_enqueue_sends)) {
+    req->rq_rpi->send_advance_fn = sender_send_env;
     LAM_SSI_RPI_GM_SEND_QUEUE_ADD(req);
 
     /* Don't bother trying to advance the queue now -- every time we
@@ -129,17 +125,68 @@ lam_ssi_rpi_gm_long_send(MPI_Request req)
      was set above (send the second part of the message). */
 
   else {
-    --lam_ssi_rpi_gm_stokens;
-    lam_debug_cond((PUB(did), "decremented stokens 1: %d", 
-                    lam_ssi_rpi_gm_stokens));
     req->rq_rpi->send_advance_fn = NULL;
-    return send_long_first(req);
+    return sender_send_env(req);
   }
 
   /* Never exit through here */
 }
 
 
+/*
+ * <SENDER>
+ *
+ * This function is a wrapper around the actual gm send -- all the
+ * setup has been done.
+ */
+static int 
+sender_send_env(MPI_Request req)
+{
+  /* Now actually send the envelope */
+
+  lam_debug_cond((PUB(did), "<long>sender_send_env"));
+  lam_ssi_rpi_gm_push_envelope(req, sender_send_env_callback);
+
+  /* Finally -- do a little latency hiding.  If we could not pin the
+     original buffer, we have already allocated a DMA-able buffer to
+     send from.  Since we just sent the first envelope, do the memcpy
+     from the original bummer -> DMA buffer now while we're waiting
+     for the envelope to be delivered.  This allows us to send
+     immediately upon receive of the ACK from the recipient. */
+
+  if (req->rq_rpi->dma_data_buf != NULL) {
+    lam_memcpy(req->rq_rpi->dma_data_buf, req->rq_packbuf, req->rq_packsize);
+  }
+
+  return 0;
+}
+
+
+/*
+ * <SENDER>
+ *
+ * Callback for when the sender envelope finishes sending.
+ */
+static void 
+sender_send_env_callback(struct gm_port *port, void *context, 
+                         gm_status_t status)
+{
+  LAM_SSI_RPI_GM_SEND_CALLBACK_START(sender_send_env_callback);
+
+  lam_debug_cond((lam_ssi_rpi_gm_did, 
+                  "<long>sender_send_env_callback: all done! %p, peer %d",
+                  req, req->rq_rank));
+
+  /* Nothing more to do -- this request has already been queued up to
+     receive an ACK */
+  /* Increment the send tokens and mark that this c2c RPI has advanced */
+
+  LAM_SSI_RPI_GM_SEND_CALLBACK_FINISH;
+}
+
+
+/************************************************************************/
+
 /* 
  * <RECEIVER>
  *
@@ -153,81 +200,37 @@ int
 lam_ssi_rpi_gm_long_receive_env_first(struct lam_ssi_rpi_gm_envl *env, 
                                       MPI_Request req)
 {
-  int ret;
-
   lam_debug_cond((lam_ssi_rpi_gm_did, 
                   "<long>long_receive_env_first: got the announce "
                   "envelope -- send back an ACK with myrank: %d",
                   req->rq_comm->c_group->g_myrank));
-
-  /* It is possible that this request was not intended to be a long,
-     and that the incoming message forced it to use the long protocol.
-     In this case, we will not yet have allocated a second envelope
-     buffer that is necessary (i.e., during the rpi_gm_start() API
-     call) -- it's an optimization that we only alloc the second env
-     buffer if we think we'll need it.  But due to a message mismatch
-     (which is a user error), we now need one.  So allocate it. */
-
-  if (req->rq_rpi->cq_envbuf2 == NULL) {
-    req->rq_rpi->cq_envbuf2 = lam_ssi_rpi_gm_dma_env_malloc();
-    if (req->rq_rpi->cq_envbuf2 == NULL)
-      return LAMERROR;
-  }
+  req->rq_rpi->is_advancing = 1;
 
   /* Prepare the ACK to be sent back.  Include my identity (relative
      to the local group in the request's communicator).  Hence, this
      works even if this is an intercommunicator, because it will be
      relative to my group. */
 
-  req->rq_rpi->cq_envbuf->ge_env.ce_flags |= C2CACK;
-  req->rq_rpi->cq_envbuf->ge_env.ce_len = LAM_min(env->ge_env.ce_len, 
-                                                  req->rq_packsize);
-  req->rq_rpi->cq_envbuf->ge_env.ce_rank = req->rq_comm->c_group->g_myrank;
-  req->rq_rpi->cq_envbuf->ge_env.ce_tag = env->ge_env.ce_tag;
-  req->rq_rpi->cq_envbuf->ge_env.ce_cid = env->ge_env.ce_cid;
+  req->rq_rpi->cq_envbuf->ge_flags |= C2CLONG | C2CACK;
+  req->rq_rpi->cq_envbuf->ge_len = LAM_min(env->ge_len, 
+                                           req->rq_packsize);
+  req->rq_rpi->cq_envbuf->ge_rank = req->rq_comm->c_group->g_myrank;
+  req->rq_rpi->cq_envbuf->ge_tag = env->ge_tag;
+  req->rq_rpi->cq_envbuf->ge_cid = env->ge_cid;
+  req->rq_rpi->cq_envbuf->ge_src_request = env->ge_src_request;
+  req->rq_rpi->cq_envbuf->ge_dest_request = req;
 
   /* Fill in the status */
   
   lam_ssi_rpi_fill_mpi_status(req, req->rq_rpi->cq_peer, 
-                              req->rq_rpi->cq_envbuf->ge_env.ce_tag, 
-                              req->rq_rpi->cq_envbuf->ge_env.ce_len);
+                              req->rq_rpi->cq_envbuf->ge_tag, 
+                              req->rq_rpi->cq_envbuf->ge_len);
   
   /* Check to see if this is going to be a truncation error. */
 
-  if (req->rq_packsize < env->ge_env.ce_len)
+  if (req->rq_packsize < env->ge_len)
     req->rq_flags |= LAM_RQFTRUNC;
 
-  /* We may need to remove this request from one or more pending
-     receive queues.  
-
-     - If this request posted a zero length receive buffer, then we
-     won't receive anything more from the <SENDER> -- so we can remove
-     it from all pending receive queues.  
-
-     - If not, then if this request was posted to receive from
-     MPI_ANY_SOURCE, then we need to remove it from the pending
-     receive queues of all other processes (just not the one that
-     we're actually receiving from). */
-
-  if (req->rq_packsize == 0) {
-    lam_debug_cond((lam_ssi_rpi_gm_did, 
-                    "<long>long_receive_env_first: zero len msg -- "
-                    "removing from queues -- src %d",
-                    req->rq_rank));
-    if (req->rq_rank == MPI_ANY_SOURCE)
-      ret = lam_ssi_rpi_gm_rm_read_any_src(req, NULL);
-    else
-      ret = lam_ssi_rpi_gm_rm_read_req(req->rq_proc->p_rpi, req);
-    if (ret != 0)
-      return LAMERROR;
-  } else if (req->rq_rank == MPI_ANY_SOURCE) {
-    lam_debug_cond((lam_ssi_rpi_gm_did, 
-                    "<long>long_receive_env_first: ANY_SOURCE, so "
-                    "rm req from all queues except %p", 
-                    req->rq_proc));
-    ret = lam_ssi_rpi_gm_rm_read_any_src(req, req->rq_proc);
-  }
-
   /* Similar to normal messages, if we don't have enough send tokens
      or if there's stuff already in the send queue, just queue it up
      for later sending.  ACKs take precedence over normal pending send
@@ -235,8 +238,9 @@ lam_ssi_rpi_gm_long_receive_env_first(struct lam_ssi_rpi_gm_envl *env,
      the ACK queue will be empty if there are any stokens available,
      so don't need to check if that's empty, either) */
 
-  if (lam_ssi_rpi_gm_stokens < 1) {
-    req->rq_rpi->send_advance_fn = send_long_ack;
+  if ((PUB(stokens) < 1) || 
+      (1 == lam_ssi_rpi_gm_enqueue_sends)) {
+    req->rq_rpi->send_advance_fn = receiver_send_ack;
     LAM_SSI_RPI_GM_ACK_QUEUE_ADD(req);
 
     /* Don't bother trying to advance the queue now -- every time we
@@ -247,43 +251,90 @@ lam_ssi_rpi_gm_long_receive_env_first(struct lam_ssi_rpi_gm_envl *env,
     return 0;
   } 
 
-  /* Otherwise, we can try to send it now.  If the function returns
-     -LAMERROR, then there weren't any long message tags available, so
-     queue this up for later. */
+  /* Otherwise, we can try to send it now. */
 
   else {
-    lam_debug_cond((lam_ssi_rpi_gm_did, 
-                    "<long>long_receive_env_first: sending ACK "
-                    "immediately"));
-    --lam_ssi_rpi_gm_stokens;
-    switch (send_long_ack(req)) {
-    case 0:
-      lam_debug_cond((PUB(did), "decremented stokens 1: %d", 
-                      lam_ssi_rpi_gm_stokens));
-      return 0;
-      break;
-
-    case -LAMERROR:
-      ++lam_ssi_rpi_gm_stokens;
-      lam_debug_cond((lam_ssi_rpi_gm_did, 
-                      "<long>long_receive_env_first: no long tags "
-                      "avail; queueing up"));
-      req->rq_rpi->send_advance_fn = send_long_ack;
-      LAM_SSI_RPI_GM_ACK_QUEUE_ADD(req);
-      return 0;
-      break;
-
-    default:
-      ++lam_ssi_rpi_gm_stokens;
-      return LAMERROR;
-      break;
-    }
+    return receiver_send_ack(req);
   }
 
   /* Won't exit out of here */
 }
 
 
+/*
+ * <RECEIVER>
+ *
+ * This is the worker function for sending the ACK back to the sender
+ * after the receiver has received the first envelope in a long send.  
+ */
+static int 
+receiver_send_ack(MPI_Request req)
+{
+  struct lam_ssi_rpi_gm_envl *env = req->rq_rpi->cq_envbuf;
+
+  /* If this request has a posted buffer length of 0, then we don't
+     need to post a receive buffer -- just send back the ACK. */
+
+  lam_debug_cond((PUB(did), "<long>receiver_send_ack"));
+  if (env->ge_len > 0) {
+
+    /* Figure out which target address to use */
+
+    if (req->rq_rpi->dma_data_buf != NULL)
+      env->ge_target = req->rq_rpi->dma_data_buf;
+    else
+      env->ge_target = req->rq_packbuf;
+  }
+
+  /* Now try to send the ACK */
+
+  lam_debug_cond((lam_ssi_rpi_gm_did, 
+                  "<long>receiver_send_ack: sending the ACK back to "
+                  "receiver, flags 0x%x", env->ge_flags));
+  lam_ssi_rpi_gm_push_envelope(req, receiver_send_ack_callback);
+  lam_ssi_rpi_gm_haveadv = 1;
+  return 0;
+}
+
+
+/*
+ * <RECEIVER>
+ * 
+ * Callback for then the ACK from the <RECEIVER> to the <SENDER>
+ * completes.
+ */
+static void 
+receiver_send_ack_callback(struct gm_port *port, void *context, 
+                           gm_status_t status)
+{
+  LAM_SSI_RPI_GM_SEND_CALLBACK_START(receiver_send_ack_callback);
+
+  lam_debug_cond((lam_ssi_rpi_gm_did,
+                  "<long>receiver_send_ack_callback: ce_len %d\n", 
+                  req->rq_rpi->cq_envbuf->ge_len));
+
+  /* Reset the req so that it can match the next incoming envelope --
+     the one that will have the actual data of the message coming
+     right behind it.  Special case, though -- if the posted request
+     had a zero length buffer, then this request is done; the sender
+     won't send the actual message (and this is a truncation error).
+     Also, we won't have posted a receive buffer, nor allocated a long
+     message tag -- so there's nothing left to do, and this request is
+     done. */
+
+  if (req->rq_rpi->cq_envbuf->ge_len == 0) {
+    req->rq_state = LAM_RQSDONE;
+    --lam_rq_nactv;
+  }
+
+  /* Increment the send tokens and mark that this RPI has advanced */
+
+  LAM_SSI_RPI_GM_SEND_CALLBACK_FINISH;
+}
+
+
+/************************************************************************/
+
 /*
  * <SENDER>
  *
@@ -309,25 +360,25 @@ lam_ssi_rpi_gm_long_receive_env_first(struct lam_ssi_rpi_gm_envl *env,
  *
  * Returns 0 or LAMERROR.
  */
-int 
-lam_ssi_rpi_gm_long_send_second(MPI_Request req, char *raw_env_data)
+static int 
+sender_recv_ack(MPI_Request req, char *raw_env_data)
 {
   struct lam_ssi_rpi_gm_envl *env = 
     (struct lam_ssi_rpi_gm_envl *) raw_env_data;
   req->rq_rpi->recv_advance_fn = NULL;
 
   lam_debug_cond((lam_ssi_rpi_gm_did,
-                  "<long>send_second: sending request %p", req));
+                  "<long>sender_recv_ack: sending request %p", req));
   lam_debug_cond((lam_ssi_rpi_gm_did, 
-                  "<long>send_second: recvd env: to rank %d, cid %d, tag %d (env tag %d), len %d, seq %d",
+                  "<long>sender_recv_ack: recvd env: to rank %d, cid %d, tag %d (env tag %d), len %d",
                   req->rq_rank, req->rq_cid, req->rq_tag, 
-                  env->ge_env.ce_tag, env->ge_env.ce_len, env->ge_env.ce_seq));
+                  env->ge_tag, env->ge_len));
 
   /* Only transfer as many bytes as the receiver requests -- which may
      be zero.  If the receiver requests zero bytes, then we are done
      -- don't send a second envelope/body pair. */
 
-  if (env->ge_env.ce_len == 0) {
+  if (env->ge_len == 0) {
     req->rq_state = LAM_RQSDONE;
     --lam_rq_nactv;
     lam_ssi_rpi_gm_haveadv = 1;
@@ -336,20 +387,20 @@ lam_ssi_rpi_gm_long_send_second(MPI_Request req, char *raw_env_data)
 
   /* Otherwise, set the envelope up for sending again */
 
-  req->rq_rpi->cq_envbuf->ge_env.ce_len = env->ge_env.ce_len;
-  req->rq_rpi->cq_envbuf->ge_env.ce_seq = env->ge_env.ce_seq;
-  req->rq_rpi->cq_envbuf->ge_env.ce_flags &= ~C2CACK;
-  req->rq_rpi->cq_envbuf->ge_env.ce_flags |= C2C2ND;
-  req->rq_rpi->cq_envbuf->ge_env.ce_rank = req->rq_comm->c_group->g_myrank;
+  req->rq_rpi->cq_envbuf->ge_len = env->ge_len;
+  req->rq_rpi->cq_envbuf->ge_flags |= C2C2ND;
+  req->rq_rpi->cq_envbuf->ge_rank = req->rq_comm->c_group->g_myrank;
   req->rq_rpi->cq_envbuf->ge_target = env->ge_target;
+  req->rq_rpi->cq_envbuf->ge_dest_request = env->ge_dest_request;
 
   /* We only need 1 send token right now because we're just going to
      push the body of the message; sending the "all done" envelope
      comes later. */
 
-  if (!LAM_SSI_RPI_GM_SEND_QUEUE_EMPTY() ||
-      lam_ssi_rpi_gm_stokens < 1) {
-    req->rq_rpi->send_advance_fn = send_long_second;
+  if (!LAM_SSI_RPI_GM_SEND_QUEUE_EMPTY() || 
+      (PUB(stokens) < 1) || 
+      (1 == lam_ssi_rpi_gm_enqueue_sends)) {
+    req->rq_rpi->send_advance_fn = sender_put_body;
     LAM_SSI_RPI_GM_SEND_QUEUE_ADD(req);
 
     /* Don't bother trying to advance the queue now -- every time we
@@ -364,55 +415,14 @@ lam_ssi_rpi_gm_long_send_second(MPI_Request req, char *raw_env_data)
      request is done when the gm_send completes. */
 
   else {
-    --lam_ssi_rpi_gm_stokens;
-    lam_debug_cond((PUB(did), "decremented stokens 1: %d", 
-                    lam_ssi_rpi_gm_stokens));
     req->rq_rpi->send_advance_fn = NULL;
-    return send_long_second(req);
+    return sender_put_body(req);
   }
 
   /* Doesn't reach here */
 }
 
 
-/* 
- * <RECEIVER>
- *
- * This function is invoked when the second envelope from the <SENDER>
- * has been received, indicating that the long message has been
- * directly DMA PUT into the target buffer, and the request is now
- * complete.
- */
-int 
-lam_ssi_rpi_gm_long_receive_env_second(struct lam_ssi_rpi_gm_envl *env, 
-                                       MPI_Request req)
-{
-  lam_debug_cond((PUB(did),
-                  "<long>receive_env_second: got final envelope!"));
-  req->rq_rpi->recv_advance_fn = NULL;
-  req->rq_proc->p_rpi->cp_current_recv_req = NULL;
-
-  /* Process the long body receive */
-
-  long_receive_body(req);
-
-  /* Since this is no longer in a queue (since it's in the
-     cp_current_recv_req), we can remove it from this proc's pending
-     receive queue.  Indeed, we *must* remove it here because the
-     upper-level receive_envelope() function doesn't understand that
-     this may be a receive from ANY_SOURCE, and therefore has already
-     been removed from all other receive queues besides the one on
-     this proc. */
-
-  if (lam_ssi_rpi_gm_rm_read_req(req->rq_proc->p_rpi, req) != 1)
-    return LAMERROR;
-
-  return 0;
-}
-
-
-/************************************************************************/
-
 /*
  * <SENDER>
  *
@@ -420,129 +430,15 @@ lam_ssi_rpi_gm_long_receive_env_second(struct lam_ssi_rpi_gm_envl *env,
  * setup has been done.
  */
 static int 
-send_long_first(MPI_Request req)
-{
-  /* Copy the envelope because we're actually sending from cq_envbuf2.
-     This is to avoid a race condition where we send the envelope and
-     the receiver returns an ACK *before* gm invokes the callback from
-     sending the envelope.  Hence cq_envbuf will be set to match the
-     incoming ACK *before* we actually send the envelope (from
-     cq_envbuf2). */
-
-  *(req->rq_rpi->cq_envbuf2) = *(req->rq_rpi->cq_envbuf);
-
-  /* Now we need to change the rank in the request's envelope to the
-     *receiver's* rank so that when the ACK arrives, and we're
-     comparing it against this request, the ce_rank will match.  Ditto
-     for the flags -- mark it with the C2CACK flag so that it matches
-     when we look for comparisons later. */
-
-  req->rq_rpi->cq_envbuf->ge_env.ce_rank = req->rq_rank;
-  req->rq_rpi->cq_envbuf->ge_env.ce_flags |= C2CACK;
-
-  /* Now actually send the envelope */
-
-  lam_debug_cond((PUB(did), "<long>send_long_first"));
-  if (lam_ssi_rpi_gm_push_envelope(req, long_send_first_env_callback) != 0)
-    return LAMERROR;
-
-  /* Finally -- do a little latency hiding.  If we could not pin the
-     original buffer, we have already allocated a DMA-able buffer to
-     send from.  Since we just sent the first envelope, do the memcpy
-     from the original bummer -> DMA buffer now while we're waiting
-     for the envelope to be delivered.  This allows us to send
-     immediately upon receive of the ACK from the recipient. */
-
-  if (req->rq_rpi->dma_data_buf != NULL) {
-    lam_memcpy(req->rq_rpi->dma_data_buf, req->rq_packbuf, req->rq_packsize);
-  }
-
-  return 0;
-}
-
-
-/*
- * <RECEIVER>
- *
- * This is the worker function for sending the ACK back to the sender
- * after the receiver has received the first envelope in a long send.  
- *
- * We first get a long message tag.  If there are none available,
- * return -LAMERROR, which will tell the caller that this should be
- * tried again later.
- *
- * If there is a tag available, post the receive buffer with that tag.
- */
-static int 
-send_long_ack(MPI_Request req)
-{
-  struct lam_ssi_rpi_gm_envl *env = req->rq_rpi->cq_envbuf;
-
-  /* If this request has a posted buffer length of 0, then we don't
-     need to get a tag or post a receive buffer -- just send back the
-     ACK. */
-
-  lam_debug_cond((PUB(did), "<long>send_long_ack"));
-  if (env->ge_env.ce_len > 0) {
-
-    /* Figure out which target address to use */
-
-    if (req->rq_rpi->dma_data_buf != NULL)
-      env->ge_target = req->rq_rpi->dma_data_buf;
-    else
-      env->ge_target = req->rq_packbuf;
-  }
-
-  /* Copy the envelope because we're actually sending from cq_envbuf2.
-     This is to avoid a race condition where we send the ACK and the
-     sender returns C2C2ND *before* gm invokes the callback from
-     sending the ACK.  Hence cq_envbuf will be set to match the
-     incoming C2C2ND *before* we actually send the ACK (from
-     cq_envbuf2). */
-
-  *(req->rq_rpi->cq_envbuf2) = *(req->rq_rpi->cq_envbuf);
-
-  /* Now setup the cq_envbuf to receive the data from the sender.
-     Reset the envelope so that it will match the incoming envelope
-     when the sender sends to me again.  */
-
-  req->rq_rpi->cq_envbuf->ge_env.ce_flags &= ~C2CACK;
-  req->rq_rpi->cq_envbuf->ge_env.ce_flags |= C2C2ND;
-  req->rq_rpi->cq_envbuf->ge_env.ce_rank = req->rq_rpi->cq_peer;
-  lam_debug_cond((lam_ssi_rpi_gm_did,
-                  "<long>long_receive_env_first: ce_flags is now 0x%x, rank is %d\n", 
-                  req->rq_rpi->cq_envbuf->ge_env.ce_flags,
-                  req->rq_rpi->cq_envbuf->ge_env.ce_rank));
-
-  /* Now try to send the ACK */
-
-  lam_debug_cond((lam_ssi_rpi_gm_did, 
-                  "<long>send_long_ack: sending the ACK back to "
-                  "receiver, flags 0x%x", env->ge_env.ce_flags));
-  if (lam_ssi_rpi_gm_push_envelope(req, long_send_ack_callback) != 0)
-    return LAMERROR;
-
-  lam_ssi_rpi_gm_haveadv = 1;
-  return 0;
-}
-
-
-/*
- * <SENDER>
- *
- * This function is a wrapper around the actual gm send -- all the
- * setup has been done.
- */
-static int 
-send_long_second(MPI_Request req)
+sender_put_body(MPI_Request req)
 {
   lam_debug_cond((PUB(did), "<long>send_long_second"));
 
   /* Do a direct DMA PUT to transfer the message to the target buffer
      on the destination node */
 
-  if (lam_ssi_rpi_gm_push_long_body(req, long_send_body_unpin_callback,
-                                    long_send_body_no_unpin_callback) != 0) {
+  if (lam_ssi_rpi_gm_put_long_body(req, sender_put_body_unpin_callback,
+                                   sender_put_body_no_unpin_callback) != 0) {
     return LAMERROR;
   }
 
@@ -552,66 +448,6 @@ send_long_second(MPI_Request req)
 }
 
 
-/*
- * <SENDER>
- *
- * Callback for when the sender envelope finishes sending.
- */
-static void 
-long_send_first_env_callback(struct gm_port *port, void *context, 
-			     gm_status_t status)
-{
-  LAM_SSI_RPI_GM_SEND_CALLBACK_START;
-
-  lam_debug_cond((lam_ssi_rpi_gm_did, 
-                  "<long>send_first_env_callback: all done! %p, peer %d",
-                  req, req->rq_rank));
-
-  /* Nothing more to do -- this request has already been queued up to
-     receive an ACK */
-  /* Increment the send tokens and mark that this c2c RPI has advanced */
-
-  LAM_SSI_RPI_GM_SEND_CALLBACK_FINISH;
-}
-
-
-/*
- * <RECEIVER>
- * 
- * Callback for then the ACK from the <RECEIVER> to the <SENDER>
- * completes.
- */
-static void 
-long_send_ack_callback(struct gm_port *port, void *context, 
-		       gm_status_t status)
-{
-  LAM_SSI_RPI_GM_SEND_CALLBACK_START;
-
-  lam_debug_cond((lam_ssi_rpi_gm_did,
-                  "<long>send_ack_callback: ce_len %d, ge_env.ce_seq %d\n", 
-                  req->rq_rpi->cq_envbuf->ge_env.ce_len,
-                  req->rq_rpi->cq_envbuf->ge_env.ce_seq));
-
-  /* Reset the req so that it can match the next incoming envelope --
-     the one that will have the actual data of the message coming
-     right behind it.  Special case, though -- if the posted request
-     had a zero length buffer, then this request is done; the sender
-     won't send the actual message (and this is a truncation error).
-     Also, we won't have posted a receive buffer, nor allocated a long
-     message tag -- so there's nothing left to do, and this request is
-     done. */
-
-  if (req->rq_rpi->cq_envbuf->ge_env.ce_len == 0) {
-    req->rq_state = LAM_RQSDONE;
-    --lam_rq_nactv;
-  }
-
-  /* Increment the send tokens and mark that this RPI has advanced */
-
-  LAM_SSI_RPI_GM_SEND_CALLBACK_FINISH;
-}
-
-
 /*
  * <SENDER>
  *
@@ -620,13 +456,13 @@ long_send_ack_callback(struct gm_port *port, void *context,
  * buffer (because it was a temporary buffer).
  */
 static void
-long_send_body_unpin_callback(struct gm_port *port, void *context, 
-			      gm_status_t status)
+sender_put_body_unpin_callback(struct gm_port *port, void *context, 
+                               gm_status_t status)
 {
-  LAM_SSI_RPI_GM_SEND_CALLBACK_START;
+  LAM_SSI_RPI_GM_SEND_CALLBACK_START(sender_put_body_unpin_callback);
 
   lam_debug_cond((lam_ssi_rpi_gm_did, 
-                  "<long>send_body_unpin_callback: all done! %p "
+                  "<long>sender_put_body_unpin_callback: all done! %p "
                   "to peer %d", 
                   req, req->rq_rank));
 
@@ -635,12 +471,13 @@ long_send_body_unpin_callback(struct gm_port *port, void *context,
      actually do a gm_send from here (because we're already in a
      callback), so queue it up to go after we finish here. */
 
-  req->rq_rpi->send_advance_fn = send_all_done_env;
+  req->rq_rpi->send_advance_fn = sender_send_all_done;
   LAM_SSI_RPI_GM_SEND_QUEUE_ADD(req);
 
   /* Unpin data buffer (there's no pool to return it to) */
 
-  lam_ssi_rpi_gm_dma_free(req->rq_rpi->dma_data_buf);
+  lam_ssi_rpi_gm_dma_free(req->rq_rpi->dma_data_buf,
+                          req->rq_packsize);
   req->rq_rpi->dma_data_buf = NULL;
 
   /* Increment the send tokens and mark that we have advanced */
@@ -657,13 +494,13 @@ long_send_body_unpin_callback(struct gm_port *port, void *context,
  * send buffer (because it was a user-pinned buffer).
  */
 static void 
-long_send_body_no_unpin_callback(struct gm_port *port, void *context, 
-				 gm_status_t status)
+sender_put_body_no_unpin_callback(struct gm_port *port, void *context, 
+                                  gm_status_t status)
 {
-  LAM_SSI_RPI_GM_SEND_CALLBACK_START;
+  LAM_SSI_RPI_GM_SEND_CALLBACK_START(sender_put_body_no_unpin_callback);
 
   lam_debug_cond((lam_ssi_rpi_gm_did, 
-                  "<long>send_body_no_unpin_callback: all done! %p "
+                  "<long>sender_put_body_no_unpin_callback: all done! %p "
                   "to peer %d", 
                   req, req->rq_rank));
 
@@ -672,14 +509,14 @@ long_send_body_no_unpin_callback(struct gm_port *port, void *context,
      actually do a gm_send from here (because we're already in a
      callback), so queue it up to go after we finish here. */
 
-  req->rq_rpi->send_advance_fn = send_all_done_env;
+  req->rq_rpi->send_advance_fn = sender_send_all_done;
   LAM_SSI_RPI_GM_SEND_QUEUE_ADD(req);
 
   /* There's no unpinning to do in this callback.  Just unuse the
-     interval */
+     interval.  Regardless of how much data we received, we originally
+     pinned req->rq_packsize.  So unuse the whole interval. */
 
-  lam_ssi_rpi_gm_interval_unuse(req->rq_packbuf,
-                                req->rq_rpi->cq_envbuf->ge_env.ce_len);
+  lam_ssi_rpi_gm_interval_unuse(req->rq_packbuf, req->rq_packsize);
 
   /* Increment the send tokens and mark that we have advanced */
 
@@ -687,6 +524,8 @@ long_send_body_no_unpin_callback(struct gm_port *port, void *context,
 }
 
 
+/************************************************************************/
+
 /*
  * <SENDER>
  *
@@ -697,30 +536,28 @@ long_send_body_no_unpin_callback(struct gm_port *port, void *context,
  * is done.
  */
 static int
-send_all_done_env(MPI_Request req)
+sender_send_all_done(MPI_Request req)
 {
-  struct lam_ssi_rpi_gm_envl *env2 = req->rq_rpi->cq_envbuf2;
-
-  /* Fake out gm_push_envelope() (below) by setting envbuf2 to NULL so
-     that it sends out from cq_envbuf (which is all setup to go). */
-
-  req->rq_rpi->cq_envbuf2 = NULL;
-
   /* Send the envelope; it's all setup already. */
 
-  if (lam_ssi_rpi_gm_push_envelope(req, long_send_second_env_callback) != 0) {
-    req->rq_rpi->cq_envbuf2 = env2;
-    return LAMERROR;
-  }
+  lam_ssi_rpi_gm_push_envelope(req, sender_send_all_done_callback);
+
+  /* 
+   * Increment the "send" bookmark to reflect the directed send of the body
+   * of the message (in _push_long_body()).  This is done here instead of
+   * immediately after the gm_directed_send* to avoid some deadlock
+   * situations.
+   *
+   * XXX: SS: is this a good place?? the other bookmark increments are all
+   * in _actions.c, so is there a clean way to move this over to _actions.c
+   * as well??
+   */
+  req->rq_proc->p_rpi->cp_out_nbytes += req->rq_rpi->cq_envbuf->ge_len;
 
   /* Nothing left to do except wait for the callback from this send */
 
   req->rq_rpi->send_advance_fn = NULL;
 
-  /* Restore envbuf2 so that it can get freed properly */
-
-  req->rq_rpi->cq_envbuf2 = env2;
-
   /* All done */
 
   return 0;
@@ -733,13 +570,13 @@ send_all_done_env(MPI_Request req)
  * Callback for when the second envelope completed sending.
  */
 static void 
-long_send_second_env_callback(struct gm_port *port, void *context, 
+sender_send_all_done_callback(struct gm_port *port, void *context, 
 			      gm_status_t status)
 {
-  LAM_SSI_RPI_GM_SEND_CALLBACK_START;
+  LAM_SSI_RPI_GM_SEND_CALLBACK_START(sender_send_all_done_callback);
 
   lam_debug_cond((lam_ssi_rpi_gm_did,
-                  "<long>send_second_env_callback: all done! %p\n", 
+                  "<long>sender_send_all_done_callback: all done! %p\n", 
                   req));
 
   /* This request is now done (this envelope was sent to notify the
@@ -755,13 +592,38 @@ long_send_second_env_callback(struct gm_port *port, void *context,
 }
 
 
+/************************************************************************/
+
+/* 
+ * <RECEIVER>
+ *
+ * This function is invoked when the second envelope from the <SENDER>
+ * has been received, indicating that the long message has been
+ * directly DMA PUT into the target buffer, and the request is now
+ * complete.
+ */
+int 
+lam_ssi_rpi_gm_long_receive_all_done(struct lam_ssi_rpi_gm_envl *env, 
+                                     MPI_Request req)
+{
+  lam_debug_cond((PUB(did),
+                  "<long>receiver_receive_all_done: got final envelope!"));
+  req->rq_rpi->recv_advance_fn = NULL;
+
+  /* Process the long body receive */
+
+  receiver_recv_body(req);
+  return 0;
+}
+
+
 /*
  * <RECEIVER>
  *
  * Called after the body of the message has actually been received.
  */
 static int
-long_receive_body(MPI_Request req)
+receiver_recv_body(MPI_Request req)
 {
   char *body;
   struct lam_ssi_rpi_gm_envl *env = req->rq_rpi->cq_envbuf;
@@ -778,20 +640,20 @@ long_receive_body(MPI_Request req)
   lam_debug_cond((lam_ssi_rpi_gm_did, 
 		  "<long>receive_long: payload len: %d -- "
                   "posted rcv for len %d from %d",
-		  env->ge_env.ce_len, req->rq_packsize, req->rq_rank));
-
-  /* Be sure to reset the cp_current_recv_req */
-
-  req->rq_proc->p_rpi->cp_current_recv_req = NULL;
+		  env->ge_len, req->rq_packsize, req->rq_rank));
 
+  /* increment "receive" bookmark to reflect this received message body */
+  req->rq_proc->p_rpi->cp_in_nbytes += env->ge_len;
+  
   /* If a temp DMA buffer was used, memcopy the data over to the real
      buffer and then unpin the temp buffer.  We know that we got only
      as many bytes as was posted in the receive -- length mismatch has
      already been handled and set in the status. */
 
   if (body != req->rq_packbuf) {
-    lam_memcpy(req->rq_packbuf, body, env->ge_env.ce_len);
-    lam_ssi_rpi_gm_dma_free(body);
+    lam_memcpy(req->rq_packbuf, body, env->ge_len);
+    lam_ssi_rpi_gm_dma_free(body, req->rq_packsize);
+    req->rq_rpi->dma_data_buf = NULL;
   }
 
   /* Regardless of how much data we received, we originally pinned
