@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2004 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2006 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,8 +12,6 @@
  * 
  * $HEADER$
  *
- * $Id: ssi_rpi_ib_actions.c,v 1.11 2004/02/18 04:15:02 vsahay Exp $
- *
  *      Function:       - calls to ib
  */
 
@@ -91,7 +89,7 @@ lam_ssi_rpi_ib_params_register(void)
   
   param_mtu = 
       lam_ssi_base_param_register_int("rpi", "ib", "mtu", NULL,
-				      MTU1024);
+				      1024);
 
   PUB(param_priority) =
       lam_ssi_base_param_register_int("rpi", "ib", "priority", NULL,
@@ -200,6 +198,7 @@ lam_ssi_rpi_ib_ib_setup(void)
       
   if (lam_ssi_rpi_verbose >= 5) {
     lam_debug(lam_ssi_rpi_did, "ib:selected port: 0x%x", PUB(port_num));
+    lam_debug(lam_ssi_rpi_did, "ib:lid: %d", PUB(lid));
     lam_debug(lam_ssi_rpi_did, "ib:tiny message length: %d bytes",
 	      PUB(tinymsglen));
     lam_debug(lam_ssi_rpi_did, "ib:mtu used is: %s", str_mtu);
@@ -1197,8 +1196,8 @@ send_peer_fc_info(struct _proc *p, PUB(fc_addr_key) addr_key)
   outmsg.nh_data[1] = upper32;
   
   lower32 = addr_key.rkey & 0xffffffff;
-  
-#if LAM_SIZEOF_VOID_P == 8
+
+#if SIZEOF_VAPI_RKEY_T == 8
   upper32 = (addr_key.rkey >> 32) & 0xffffffff;
 #else
   upper32 = 0;
@@ -1240,7 +1239,7 @@ send_peer_fc_info(struct _proc *p, PUB(fc_addr_key) addr_key)
   /* Get the rkey */
   p->p_rpi->fc_rkey = inmsg.nh_data[3];
 
-#if LAM_SIZEOF_VOID_P == 8
+#if SIZEOF_VAPI_RKEY_T == 8
   p->p_rpi->fc_rkey <<= 32;
 #endif
 
