@@ -10,11 +10,12 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *	Ohio Trollius
  *	Copyright 1997 The Ohio State University
  *	GDB
  *
- *	$Id: lpdetach.c,v 6.5.2.1 2002/10/09 19:49:54 brbarret Exp $
+ *	$Id: lpdetach.c,v 6.8 2002/10/09 21:00:46 brbarret Exp $
  *
  *	Function:	- detaches caller from the local kenya server
  *			- based on Trollius 2.0 Copyright 1990
@@ -25,6 +26,9 @@
  *	Returns:	- 0 or ERROR
  */
 
+#include <lam_config.h>
+
+#include <etc_misc.h>
 #include <events.h>
 #include <kio.h>
 #include <ksignal.h>
@@ -59,8 +63,8 @@ int			status;
 	LAM_ZERO_ME(reply);
 
 	request.pq_src_node = ltot(LOCAL);
-	request.pq_src_event = ltot((int4) -getpid());
-	request.pq_pid = ltot(pid ? (int4) pid : (int4) getpid());
+	request.pq_src_event = ltot((int4) -lam_getpid());
+	request.pq_pid = ltot(pid ? (int4) pid : (int4) lam_getpid());
 	request.pq_status = ltot((int4) status);
 	request.pq_req = ltot(PQDETACH);
 
@@ -77,7 +81,7 @@ int			status;
 		return(ERROR);
 	}
 
-	nhead.nh_event = -getpid();
+	nhead.nh_event = -lam_getpid();
 	nhead.nh_flags = 0;
 	nhead.nh_length = sizeof(reply);
 	nhead.nh_msg = (char *) &reply;
@@ -93,7 +97,7 @@ int			status;
 		errno = ttol(reply.pr_reply);
 		return(ERROR);
 	} else {
-		_kio.ki_rtf &= ~RTF_KENYA;
+		_kio.ki_rtf &= ~RTF_KENYA_ATTACH;
 		return(0);
 	}
 }
