@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2006 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -31,6 +31,7 @@
 #include <lam-ssi.h>
 #include <lam-ssi-boot.h>
 #include <sfh.h>
+#include <laminternal.h>
 
 
 /*
@@ -53,6 +54,8 @@ main(int argc, char *argv[])
   char **cmdv = NULL;
   int cmdc;
   int cmdv0_len;
+  char *prefix = NULL;
+  char *suffix = NULL;
 
   /* Parse the command line */
 
@@ -127,6 +130,20 @@ main(int argc, char *argv[])
     return (0);
   }
 
+  /* Batch system gorp */
+  if (ao_taken(ad, "sessionprefix")) {
+    prefix = ao_param(ad, "sessionprefix", 0, 0);
+  } else {
+    prefix = NULL;
+  }
+
+  if (ao_taken(ad, "sessionsuffix")) {
+    suffix = ao_param(ad, "sessionsuffix", 0, 0);
+  } else {
+    suffix = NULL;
+  }
+  lam_tmpdir_init_opt(prefix, suffix, 0);
+
   /* Compatability arguments, from pre-SSI days (and it's just a
      heckuva lot shorter to type!) */
 
